#!/usr/bin/env python3

class RpqLua:
    def __init__(self, redisConnection):
        '''Sets Redis connection, load LUA from path'''

        self.setRedisConnection(redisConnection);
        self.loadSource(self.getLuaPath());

    def getLuaPath(self):
        """
            Returns the LUA path in the filesystem
        """

        import rpq_src, pkg_resources

        return pkg_resources.resource_filename('rpq_src', 'redis-priority-queue.lua');

    def file_get_contents(self, filename):
        '''Get a file content'''

        with open(filename) as f:
            return f.read()

    def setRedisConnection(self, connection):
        '''Set a Redis connection'''

        self.connection = connection;

    def loadSource(self, path):
        '''Load LUA script source code from a file'''

        self.source = self.file_get_contents(path);

    def register(self):
        '''Load the script into Redis'''

        self.queue = self.connection.register_script(self.source)
        return self.queue;



#!/usr/bin/env python3

# redis-priority-queue -> queue monitor
# Author: Gabriel Bordeaux (gabfl)
# Github: https://github.com/gabfl/redis-priority-queue
# Compatible with python 2.7 & 3

import redis, argparse, json
from prettytable import PrettyTable

# Parse arguments
parser = argparse.ArgumentParser()
parser.add_argument("-H", "--host", default="127.0.0.1",
                    help="Redis server host")
parser.add_argument("-p", "--port", type=int, default=6379,
                    help="Redis server port")
parser.add_argument("-a", "--auth",
                    help="Redis server authentification")
parser.add_argument("-n", "--dbnum", type=int, default=0,
                    help="Redis server database number")
parser.add_argument("-s", "--sort_groups",
                    help="Sort groups", type=json.loads)
args = parser.parse_args()

# Global vars
t = None
r = None

def setSortGroups(sortGroups = None):
    """Return the sorting groups, either user defined or from the default list"""
    if sortGroups is None: # Default groups
        return [('-inf', '+inf'), ('-inf', 100), (101, '+inf')];
    else:
        sortGroups.insert(0, ('-inf', '+inf'))
        return sortGroups;

def getCount(queueName, min, max):
    """Fetches set count from Redis"""

    global r

    return r.zcount(queueName, min, max);

def getColumnTitle(min, max):
    """Human readable column titles"""

    if str(min) == '-inf' and str(max) == '+inf':
        return 'Total';
    elif str(min) == '-inf':
        return 'Up to ' + '{0:,}'.format(max);
    elif str(max) == '+inf':
        return 'From ' + '{0:,}'.format(min);
    elif min == max:
        return '{0:,}'.format(min);
    else:
        return '{0:,}'.format(min) + ' to ' + '{0:,}'.format(max);

def setColumnAlign(titles):
    """Set PrettyTable column alignment"""

    global t

    for i, title in enumerate(titles):
        if i == 0: # First column (title)
            t.align[title] = 'l'; # Left
        else: # Any other column
            t.align[title] = 'r'; # Right

def main():
    global t, r

    # Redis connection
    try:
        r = redis.StrictRedis(host = args.host, port = args.port, db = args.dbnum, password = args.auth)
    except Exception as e:
        import sys

        print('Redis error: %s' % (e))
        sys.exit()

    # Sort groups
    sortGroups = setSortGroups(args.sort_groups);

    # Column titles (queue name, then a column per sorting group)
    titles = ['Queue name'];
    titles.extend([getColumnTitle(sortGroup[0], sortGroup[1]) for sortGroup in sortGroups]);

    # Create table
    t = PrettyTable(titles);
    setColumnAlign(titles);

    # Get queues
    try:
        queueNames = r.smembers('rpq|names')
    except Exception as e:
        import sys

        print('Redis error: %s' % (e))
        sys.exit()

    # Add a row par queue
    for queueName in sorted(queueNames):
        # Get row
        row = [queueName.decode("utf-8")]
        row.extend(['{0:,}'.format(getCount(queueName, sortGroup[0], sortGroup[1])) for sortGroup in sortGroups]);

        # Add row
        t.add_row(row);

    # Print table
    print (t);

if __name__ == '__main__':
    main()



#!/usr/bin/env python3

class RpqQueue:
    def __init__(self, queue):
        '''set RpqLua registered queue'''
        self.setRegisteredQueue(queue);

    def bytesDecode(self, item):
        '''Decode Bytes to utf-8 for Python 3 compatibility'''
        return item.decode("utf-8");

    def setRegisteredQueue(self, queue):
        '''set RpqLua registered queue'''
        self.queue = queue;

    def setqueueName(self, queueName):
        '''Define loaded queue name at class level'''
        self.queueName = queueName;

    def push(self, item, priority = None):
        '''Push an item'''
        # Build arg list
        args = ['push', self.queueName, item]
        if priority is not None:
            args.append(priority);

        return self.queue(args = args)

    def pop(self, orderBy = 'desc'):
        '''alias for popOne()'''
        return self.popOne(orderBy)

    def popOne(self, orderBy = 'desc'):
        '''Pop an item'''
        item = self.popMany(orderBy, 1)

        if item: # There is an item
            return item[0];
        else:
            return None;

    def popMany(self, orderBy = 'desc', numberOfItems = 1):
        '''Pop many items'''
        items = self.queue(args = ['pop', self.queueName, orderBy, numberOfItems])
        if items:
            # Decode all items from bytes to utf-8
            items = tuple(map(self.bytesDecode, items));

            return items;
        else:
            return None;

    def peek(self, orderBy = 'desc'):
        '''alias for peekOne()'''
        return self.peekOne(orderBy)

    def peekOne(self, orderBy = 'desc'):
        '''Peek an item'''
        item = self.peekMany(orderBy, 1)

        if item: # There is an item
            return item[0];
        else:
            return None;

    def peekMany(self, orderBy = 'desc', numberOfItems = 1):
        '''Peek many items'''
        items = self.queue(args = ['peek', self.queueName, orderBy, numberOfItems])
        if items:
            # Decode all items from bytes to utf-8
            items = tuple(map(self.bytesDecode, items));

            return items;
        else:
            return None;

    def count(self, priorityMin = None, priorityMax = None):
        '''Get queue size'''
        # Build arg list
        args = ['size', self.queueName]
        if priorityMin is not None:
            args.append(priorityMin);
            if priorityMax is not None:
                args.append(priorityMax);

        return self.queue(args = args)



from setuptools import setup

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except(IOError, ImportError):
    long_description = open('README.md').read()

setup (
    name = 'rpq',
    version = '1.1',
    description = 'Simple Redis work queue with added features (priorities, pop multiple items at once)',
    long_description = long_description,
    author = 'Gabriel Bordeaux',
    author_email = 'pypi@gab.lc',
    url = 'https://github.com/gabfl/redis-priority-queue',
    license = 'MIT',
    packages = ['rpq', 'rpq_src'],
    package_dir = { 'rpq': 'clients/python/lib', 'rpq_src': 'src' },
    package_data={
      'rpq_src': ['*.lua'],
   },
    install_requires = ['redis', 'argparse', 'prettytable'], # external dependencies
    entry_points = {
        'console_scripts': [
            'rpq_monitor = rpq_src.queue_monitor:main',
        ],
    },
)
